﻿using System;
using System.Collections.Generic;
using System.Text;

namespace ConsoleGraph
{
    public static class WindowManager
    {

        #region Private Field

        private static ConsoleColor _bg;
        private static ConsoleColor _fg;
        private static int startX;
        private static int startY;
        private static ConsoleColor sForegroundColour;
        private static ConsoleColor sBackgroundColour;
        private static int sBufferHeight;
        private static int sBufferWidth;

        #endregion


        /// <summary xml:lang="it">
        /// Imposta il titolo di una finestra
        /// </summary>
        /// <summary xml:lang="en">
        /// Set Window Console Title
        /// </summary>
        /// <param name="text">Text for Application</param>
        public static void Title(string text)
        {
            Console.Title = text;
        }



        /// <summary xml:lag="it">
        /// Salva lo stato dei colori
        /// </summary>
        /// <summary xml:lang="en">
        /// Save Color State
        /// </summary>
        private static void SaveColorState()
        {
            _bg = Console.BackgroundColor;
            _fg = Console.ForegroundColor;
        }



        /// <summary xml:lang="it">
        /// Ripristina lo stato dei colori
        /// </summary>
        /// <summary xml:lang="en">
        /// Restore Color State
        /// </summary>
        private static void RestoreColorState()
        {
            Console.BackgroundColor = _bg;
            Console.ForegroundColor = _fg;
        }



        /// <summary xml:lang="it">
        /// Costruice la finestra base
        /// </summary>
        /// <summary xml:lang="en">
        /// Construct base window
        /// </summary>
        public static void SetupWindow()
        {
            sBufferHeight = Console.BufferHeight;
            var whereToMove = Console.CursorTop + 1; //Move one line below visible

            if (whereToMove < Console.WindowHeight) //If cursor is not on bottom line of visible
                whereToMove = Console.WindowHeight + 1;

            if (Console.BufferHeight < whereToMove + Console.WindowHeight) //Buffer is too small
                Console.BufferHeight = whereToMove + Console.WindowHeight;

            Console.MoveBufferArea(0, 0, Console.WindowWidth, Console.WindowHeight, 0, whereToMove);

            Console.CursorVisible = false;
            startX = Console.CursorTop;
            startY = Console.CursorLeft;

            sForegroundColour = Console.ForegroundColor;
            sBackgroundColour = Console.BackgroundColor;

            Console.CursorTop = 0;
            Console.CursorLeft = 0;
        }



        /// <summary xml:lang="it">
        /// Imposta nuovi parametri per la finestra
        /// </summary>
        /// <summary xml:lang="en">
        /// Setup new param for the window
        /// </summary>
        /// <param name="width">Width</param>
        /// <param name="height">Height</param>
        public static void UpdateWindow(int width, int height)
        {
            Console.CursorVisible = false;

            Console.BackgroundColor = ConsoleColor.Blue;
            Console.Clear();


            if (width > Console.BufferWidth) 
            {
                Console.BufferWidth = width;
                Console.WindowWidth = width;
            }
            else
            {
                Console.WindowWidth = width;
                Console.BufferWidth = width;
            }

            if (height > Console.BufferWidth) 
            {
                Console.BufferHeight = height;
                Console.WindowHeight = height;
            }
            else
            {
                Console.WindowHeight = height;
                Console.BufferHeight = height;
            }

            WindowManager.DrawBlock(Console.BackgroundColor, 0, 0, Console.WindowHeight, Console.WindowWidth); //Flush Buffer


        }




        /// <summary xml:lang="it">
        /// Completa la finestra
        /// </summary>
        /// <summary xml:lang="en">
        /// End the window
        /// </summary>
        public static void EndWindow()
        {
            Console.ForegroundColor = sForegroundColour;
            Console.BackgroundColor = sBackgroundColour;

            var whereToGet = startX + 1; //Move one line below visible
            if (whereToGet < Console.WindowHeight) //If cursor is not on bottom line of visible
                whereToGet = Console.WindowHeight + 1;
            Console.MoveBufferArea(0, whereToGet, Console.WindowWidth, Console.WindowHeight, 0, 0);

            Console.CursorTop = startX;
            Console.CursorLeft = startY;

            Console.CursorVisible = true;
            Console.BufferHeight = sBufferHeight;

        }



        /// <summary xml:lang="it">
        /// Scrive del testo in una determinata posizione
        /// </summary>
        /// <summary xml:lang="en">
        /// Write text in coordinates
        /// </summary>
        /// <param name="text">Text</param>
        /// <param name="startX">X</param>
        /// <param name="startY">Y</param>
        /// <param name="textColour">Text Color</param>
        /// <param name="backgroundColour">Background Color</param>
        public static void WirteText(String text, int startX, int startY, ConsoleColor textColour, ConsoleColor backgroundColour = ConsoleColor.Blue)
        {
            SaveColorState();

            Console.CursorLeft = startY;
            Console.CursorTop = startX;

            Console.BackgroundColor = backgroundColour;
            Console.ForegroundColor = textColour;

            Console.Write(text);

            RestoreColorState();


        }


        public static void DrawBlock(ConsoleColor colour, int startX, int startY, int endX, int endY)
        {

            SaveColorState();
            Console.BackgroundColor = colour;

            for (var i = startX; i < endX; i++)
            {
                Console.CursorLeft = startY;
                Console.CursorTop = i;

                Console.WriteLine("".PadLeft(endY - startY));
            }

            RestoreColorState();
        }

    }
}
